/***************************************************************
 *                 Mathematical Object Library                 *
 *	      class matrix:  declarations for matrix	       *
 *                    simula.plus@cemes.fr                     *
 *	             GNU/linux version 3.4.0                   *
 *            software under General Public License            *
 ***************************************************************
 * copyright © 1995,1996,1999,2000,2001,2002,2003,2004,2005,2006,2007,2008,2009,2012,2013 COLLARD Christophe
 * copyright © 2002,2003,2004 CREUSE Emmanuel
 * copyright © 2003,2004 MERCIER Denis
 * copyright © 1999,2000,2001,2002,2003,2004,2005,2006,2007,2008,2009,2012,2013 Centre National de la Recherche Scientifique
 * copyright © 2002,2003,2004,2005,2006,2007,2008,2009 Arts et Métiers ParisTech
 * copyright © 1999,2000,2001,2002,2003,2004,2005,2006,2007 Université de Valenciennes et du Hainaut Cambrésis
 * copyright © 2002,2003,2004,2005,2006,2007,2008,2009 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
 * copyright © 1999,2000,2001,2002,2003,2004,2005,2006,2007 Laboratoire de Mathématiques et ses Applications de Valenciennes (LAMAV)
 * copyright © 2012,2013 Centre d'Elaboration de Matériaux et d'Etudes Structurales (CEMES - CNRS)
 ***************************************************************/

/*! \namespace mol
    \brief Mathematical Object Libraries
*/

/*! \class mol::matrix
    \brief matrix library \n

    \htmlonly 
    <FONT color="#838383">

    matrix belongs to Mathematical Object Libraries (MOL++) </br>
    MOL++ is part of Simula+ <br><br>

    Simula+ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version. <br><br>

    Simula+ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. <br><br>

    You should have received a copy of the GNU General Public License
    along with Simula+; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    </FONT>
    \endhtmlonly

    \authors copyright \htmlonly &#169; \endhtmlonly 1995, 1996, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2012, 2013 Christophe COLLARD \n
             copyright \htmlonly &#169; \endhtmlonly 2002, 2003, 2004 Emmanuel CREUSE \n
	     copyright \htmlonly &#169; \endhtmlonly 2003, 2004 Denis MERCIER \n
             copyright \htmlonly &#169; 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2012, 2013 Centre National de la Recherche Scientifique \endhtmlonly \n
	     copyright \htmlonly &#169; 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Arts et M&#233;tiers ParisTech \endhtmlonly \n
	     copyright \htmlonly &#169; 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007 Universit&#233; de Valenciennes et du Hainaut Cambr&#233;sis \endhtmlonly \n
	     copyright \htmlonly &#169; 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
	     copyright \htmlonly &#169; 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007 Laboratoire de Math&#233;matiques et ses Applications de Valenciennes (LAMAV) \endhtmlonly
	     copyright \htmlonly &#169; 2012, 2013 Centre d'Elaboration de Mat&#233;riaux et d'Etudes Structurales (CEMES - CNRS) \endhtmlonly \n
    \version 3.4.0
    \date 1995-2019
    \bug none
    \warning none
*/

#ifndef __cplusplus
#error Must use C++ for the type matrix
#endif

#ifndef __matrix_hpp
#define __matrix_hpp


#ifndef __iostream
#include <iostream>
#endif

#ifndef __fstream
#include <fstream>
#endif

#ifndef __assert_h
#include <assert.h>
#endif

#ifndef __math_h
#include <math.h>
#endif

#ifndef __parameters_h
#include "parameters.h"
#endif

#ifndef __maths_hpp
#include "MOL++/maths.hpp"
#endif

#ifndef __vectors_hpp
#include "MOL++/vectors.hpp"
#endif

using namespace std;

namespace mol
{


//=============================
template <class T> class matrix
//=============================
{
  // matrix are stored with row vectors

  private:

  protected:
    int nbRows, nbColumns;  // number of rows & columns of the matrix
    vector<T> *mrows;

  public:
    matrix (); 			        // default constructor
    matrix (int, int, bool=true, T=0);  // this constructor allocates memory
    matrix (const matrix<T>&);          // copy constructor
    matrix (matrix<T>*);                // copy constructor for temporary objects
    ~matrix (); 		        // destructor

    virtual int Rows ()    const {return nbRows;}        // returns the # of rows
    virtual int Columns () const {return nbColumns;}     // returns the # of columns
    virtual void create (int,int);  // partial memory allocation (don't use this function except if you know what you're doing !!)
    virtual void assign (int,int);  // allocates memory without initilizing the matrix components to zero (so use it with great care)
    friend bool operator ! (const matrix<T>& a) {return !a.nbRows;}

    virtual T&   operator () (int,int) const;
    virtual vector<T>& operator [] (int) const;        // returns a whole row
    virtual matrix<T>& operator = (const matrix<T>&);  // defines equal operator for matrix
    virtual matrix<T>& operator = (matrix<T>*);        // allocates the data from the right hand temporary object to the left hand object - no copy (deletes temporary object)
    virtual matrix<T>& operator &= (const matrix<T>&); // copies matrix without size check (use this operator with great care)
    virtual matrix<T>& operator &= (matrix<T>*);       // allocates the data from the right hand temporary object to the left hand object (deletes temporary object) - no size check - no copy (use this operator with great care)
    template <class Tf> friend matrix<Tf> operator + (const matrix<Tf>&, const matrix<Tf>&); // overloads matrix + matrix
    template <class Tf> friend matrix<Tf> operator - (const matrix<Tf>&, const matrix<Tf>&); // overloads matrix - matrix
    template <class Tf> friend matrix<Tf> operator * (const matrix<Tf>&, const matrix<Tf>&); // overloads matrix * matrix
    template <class Tf> friend matrix<Tf> operator * (const Tf&, const matrix<Tf>&);         // overloads scalar * matrix
    template <class Tf> friend matrix<Tf> operator * (const matrix<Tf>&, const Tf&);         // overloads matrix * scalar
    template <class Tf> friend vector<Tf> operator * (const matrix<Tf>&, const vector<Tf>&); // overloads matrix * vector
    template <class Tf> friend vector<Tf> operator * (const vector<Tf>&, const matrix<Tf>&); // overloads vector * matrix
    template <class Tf> friend matrix<Tf> operator / (const matrix<Tf>&, const matrix<Tf>&); // overloads matrix / matrix
    template <class Tf> friend matrix<Tf> operator / (const Tf&, const matrix<Tf>&);         // overloads scalar / matrix
    template <class Tf> friend matrix<Tf> operator / (const matrix<Tf>&, const Tf&);         // overloads scalar / matrix
    template <class Tf> friend Tf         operator | (const matrix<Tf>&, const matrix<Tf>&); // A|B = Aij Bij (for matrix & tensor2)
    template <class Tf> friend matrix<Tf> operator ^ (const vector<Tf>&, const vector<Tf>&); // overloads operator ^ for matrix = vector*t(vector) (tensorial product for 2 vectors)
    matrix<T>& operator += (const matrix<T>&);  // overloads += operator for matrix
    matrix<T>& operator -= (const matrix<T>&);  // overloads -= operator for matrix
    matrix<T>& operator *= (const matrix<T>&);  // overloads *= operator for matrix
    matrix<T>& operator *= (const T&);          // overloads *= operator between matrix and scalar
    matrix<T>& operator /= (const matrix<T>&);  // overloads /= operator for matrix
    matrix<T>& operator /= (const T&);	        // overloads /= operator between matrix and scalar
    template <class Tf> friend bool operator == (const matrix<Tf>&, const matrix<Tf>&);  // compares 2 matrices
    template <class Tf> friend bool operator != (const matrix<Tf>&, const matrix<Tf>&);
    template <class Tf> friend ostream& operator << (ostream&, const matrix<Tf>&);       // overloads output stream for matrix
    template <class Tf> friend istream& operator >> (istream&, const matrix<Tf>&);       // overloads input stream for matrix
    template <class Tf> friend matrix<Tf> Id2 (int);                // returns identity matrix
    template <class Tf> friend Tf det (const matrix<Tf>&);          // computes the determinant operator for a matrix
    template <class Tf> friend matrix<Tf> t (const matrix<Tf>&);    // defines tranposition operator for matrix
    virtual matrix<T> inv ();	                                    // computes inversion operator for matrix
    template <class Tf> friend matrix<Tf> gauss (matrix<Tf>);       // computes matrix inversion with gauss algorithm
    template <class Tf> friend bool inv (matrix<Tf>&, long double); // same function as gauss but returns the inversion result 0/1
    //    friend matrix<T> gauss_pt <T> (matrix<T>);              // computes matrix inversion with gauss algorithm

    // i/o disk
    virtual void save (const char*);
    virtual void save (const string&);
    virtual void read (const char*);
    virtual void read (const string&);

    // operations on matrix
    template <class Tf> friend matrix<Tf> kronecker (int); // return the identity matrix (ie. i==j -> 1 and i!=j -> 0)
    matrix<T>& approximation ();
    virtual void permute_rows (int,int);     // permutes matrix rows
    virtual void permute_columns (int,int);  // permutes matrix columns
    template <class Tf> friend matrix<Tf> abs (const matrix<Tf>&);  // gives the absolute value of a matrix
    template <class Tf> friend Tf max (const matrix<Tf>&);  // gives the greatest element of a matrix
    template <class Tf> friend Tf tr (const matrix<Tf>&);   // gives the trace of a matrix
    template <class Tf> friend vector<Tf> diagonal (const matrix<Tf>&);
    template <class Tf> friend matrix<Tf> submatrix (const matrix<Tf>&, int, int, int, int);
    template <class Tf> friend matrix<Tf> merge_rows (const matrix<Tf>&, const matrix<Tf>&);
    template <class Tf> friend matrix<Tf> merge_columns (const matrix<Tf>&, const matrix<Tf>&);
    T p_norm (int); // computes p-norm for matrix
};


//=====Private methods for matrix=============================================


//=====Public methods for matrix==============================================


//-------------------------------------
template <class T> matrix<T>::matrix ()
//-------------------------------------
{
  nbRows = nbColumns = 0;
}


//------------------------------------------------------------------------------
template <class T> matrix<T>::matrix (int rows, int columns, bool init, T value)
//------------------------------------------------------------------------------
{
  // computes matrix rows as vectors
  assert (rows>0 && columns>0);

  nbColumns = columns;
  mrows = new vector<T> [nbRows=rows];
  for (int i=0; i<nbRows; i++)
    mrows[i] =& vector<T> (nbColumns,init,value);
}


//---------------------------------------------------------
template <class T> matrix<T>::matrix (const matrix<T>& mat)
//---------------------------------------------------------
{
  assert (mat.nbRows && mat.nbColumns);

  mrows = new vector<T> [nbRows=mat.nbRows]; // allocates memory for rows (no initialization)
  nbColumns = mat.nbColumns;

  for (int i=1; i<=mat.nbRows; i++)
   { mrows[i-1] = nbColumns;    // allocates memory for columns (no initialization)
     for (int j=1; j<=nbColumns; j++)
       (*this)(i,j) = mat(i,j); // allocates element by element for symmatrix compatility
   }
}


//---------------------------------------------------
template <class T> matrix<T>::matrix (matrix<T>* mat)
//---------------------------------------------------
{
  assert ((*mat)[1].dim() == (*mat).nbColumns); // forbids the acces to symmetric matrices

  nbRows = (*mat).nbRows;                // copy the size of the temporary objet
  nbColumns = (*mat).nbColumns;          // copy the size of the temporary objet
  mrows = (*mat).mrows;                  // copy the address of the temporary objet
  (*mat).nbRows = (*mat).nbColumns = 0;  // set temporary object size to zero (so the data stored at p won't be delete by the destructor)
}


//--------------------------------------
template <class T> matrix<T>::~matrix ()
//--------------------------------------
{
  // destruction of the memory allocated for the matrix

  if (nbRows) delete [] mrows; // free memory only if it's been affected
  nbRows = nbColumns = 0;
}


//-----------------------------------------------------------------
template <class T> void matrix<T>::create (int nrows, int ncolumns)
//-----------------------------------------------------------------
{
  assert (nrows>0 && ncolumns>0);
  assert (!nbRows);

  mrows = new vector<T> [nbRows = nrows];  // allocates memory for rows only
  nbColumns = ncolumns;
}


//-----------------------------------------------------------------
template <class T> void matrix<T>::assign (int nrows, int ncolumns)
//-----------------------------------------------------------------
{
  assert (nrows>0 && ncolumns>0);
  assert (!nbRows);

  nbColumns = ncolumns;
  mrows = new vector<T> [nbRows = nrows];  // allocates memory with the = operator for vector (no initialization)
  for (int i=0; i<nbRows; i++)
    mrows[i] =& vector<T> (nbColumns,false);
}


//----------------------------------------------------------------------
template <class T> T& matrix<T>::operator () (int row, int column) const
//----------------------------------------------------------------------
{
  assert ((row>0)&&(row<=nbRows));
  assert ((column>0)&&(column<=nbColumns));
  return mrows[row-1][column];
}


//------------------------------------------------------------------
template <class T> vector<T>& matrix<T>::operator [] (int row) const
//------------------------------------------------------------------
{
  assert ((row>0)&&(row<=nbRows));
  return mrows[row-1];
}


//------------------------------------------------------------------------
template <class T> matrix<T>& matrix<T>::operator = (const matrix<T>& mat)
//------------------------------------------------------------------------
{
  assert (mat.nbRows && mat.nbColumns);

  if (!nbRows) assign(mat.nbRows,mat.nbColumns);
  else assert ( (nbRows==mat.nbRows) && (nbColumns==mat.nbColumns) );

  for (int i=1; i<=mat.nbRows; i++)
    for (int j=1; j<=nbColumns; j++)
      (*this)(i,j) = mat(i,j);

  return *this;
}


//------------------------------------------------------------------
template <class T> matrix<T>& matrix<T>::operator = (matrix<T>* mat)
//------------------------------------------------------------------
{
  assert ((*mat).nbRows && (*mat).nbColumns);
  assert ((*mat)[1].dim() == (*mat).nbColumns); // forbids the acces to symmetric matrices

  if (nbRows)
    { assert ( (nbRows == (*mat).nbRows) && (nbColumns == (*mat).nbColumns) );
      delete [] mrows;
    }
  else
    { nbRows = (*mat).nbRows;
      nbColumns = (*mat).nbColumns;
    }
  mrows = (*mat).mrows;
  (*mat).nbRows = (*mat).nbColumns = 0;

  return *this;
}


//-------------------------------------------------------------------------
template <class T> matrix<T>& matrix<T>::operator &= (const matrix<T>& mat)
//-------------------------------------------------------------------------
{
  if (!mat)
    { if (nbRows) delete [] mrows;
      nbRows = nbColumns = 0;
    }

  else
    { assert (mat[1].dim() == mat.nbColumns); // forbids the acces to matrix & spmatrix
      if (nbRows != mat.nbRows)
	{ if (nbRows) delete [] mrows;
	  mrows = new vector<T> [nbRows = mat.nbRows];  // allocates memory with the = operator for vector (no initialization)
	}
      nbColumns = mat.nbColumns;
      for (int i=1; i<=mat.nbRows; i++)
	(*this)[i] &= mat[i];
    }

  return *this;
}


//-------------------------------------------------------------------
template <class T> matrix<T>& matrix<T>::operator &= (matrix<T>* mat)
//-------------------------------------------------------------------
{
  if (!(*mat).nbRows)
    { if (nbRows) delete [] mrows;
      nbRows = nbColumns = 0;
    }

  else
    { assert ((*mat)[1].dim() == (*mat).nbColumns); // forbids the acces to symmetric matrices in order to avoid matrix &= symmatrix
      if (nbRows) delete [] mrows;
      nbRows = (*mat).nbRows;
      nbColumns = (*mat).nbColumns;
      mrows = (*mat).mrows;
      (*mat).nbRows = (*mat).nbColumns = 0;
    }

  return *this;
}


//----------------------------------------------------------------------------------------
template <class Tf> matrix<Tf> operator + (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
//----------------------------------------------------------------------------------------
{
  assert ((mat1.nbRows==mat2.nbRows) && (mat1.nbColumns==mat2.nbColumns)); // matrices must have the same size
  assert((mat1.nbRows)&&(mat1.nbColumns));

  matrix<Tf> mat = mat1;
  mat += mat2;

  return mat;
}


//----------------------------------------------------------------------------------------
template <class Tf> matrix<Tf> operator - (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
//----------------------------------------------------------------------------------------
{
  assert ((mat1.nbRows==mat2.nbRows) && (mat1.nbColumns==mat2.nbColumns)); // matrices must have the same size
  assert((mat1.nbRows)&&(mat1.nbColumns));

  matrix<Tf> mat = mat1;
  mat -= mat2;

  return mat;
}


//----------------------------------------------------------------------------------------
template <class Tf> matrix<Tf> operator * (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
//----------------------------------------------------------------------------------------
{
  assert (mat1.nbColumns==mat2.nbRows); // matrix can't be multiplicated

  matrix<Tf> mt(mat1.nbRows,mat2.nbColumns); // allocates memory and set matrix components to zero

  for (int i=1; i<=mat1.nbRows; i++)
    for (int j=1; j<=mat2.nbColumns; j++)
      for (int k=1; k<=mat2.nbRows; k++)
  	mt(i,j) += mat1(i, k) * mat2(k, j);

  return mt;
}


//------------------------------------------------------------------------------
template <class Tf> matrix<Tf> operator * (const matrix<Tf>& mat, const Tf& elt)
//------------------------------------------------------------------------------
{
  assert (mat.nbRows && mat.nbColumns);
  matrix<Tf> mp = mat;
  mp *= elt;
  return mp;
}


//------------------------------------------------------------------------------
template <class Tf> matrix<Tf> operator * (const Tf& elt, const matrix<Tf>& mat)
//------------------------------------------------------------------------------
{
  assert (mat.nbRows&&mat.nbColumns);
  matrix<Tf> mp = mat;
  mp *= elt;
  return mp;
}


//------------------------------------------------------------------------------------
template <class Tf> vector<Tf> operator * (const matrix<Tf>& mat, const vector<Tf>& v)
//------------------------------------------------------------------------------------
{ 
  assert (mat.nbColumns == v.dim()); // checks validity of the operation

  vector<Tf> u(mat.nbRows); // allocates memory and set vector components to zero
  for (int i=1; i<=mat.nbRows; i++)
      for (int j=1; j<=mat.nbColumns;j++)
           u[i] += mat(i,j) * v[j];

  return u;
}


//------------------------------------------------------------------------------------
template <class Tf> vector<Tf> operator * (const vector<Tf>& v, const matrix<Tf>& mat)
//------------------------------------------------------------------------------------
{ 
  assert (mat.nbRows == v.dim());

  vector<Tf> u(mat.nbColumns); // allocates memory and set vector components to zero
  for (int i=1; i<=mat.nbColumns; i++)
      for (int j=1; j<=mat.nbRows; j++)
          u[i] += v[j] * mat(j,i);

  return u;
}


//----------------------------------------------------------------------------------------
template <class Tf> matrix<Tf> operator / (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
//----------------------------------------------------------------------------------------
{
  matrix<Tf> mt = mat1;
  mt *= gauss(mat2);

  return mt;
}


//------------------------------------------------------------------------------
template <class Tf> matrix<Tf> operator / (const Tf& elt, const matrix<Tf>& mat)
//------------------------------------------------------------------------------
{
  matrix<Tf> mt = gauss(mat);
  mt*= elt;

  return mt;
}


//------------------------------------------------------------------------------
template <class Tf> matrix<Tf> operator / (const matrix<Tf>& mat, const Tf& elt)
//------------------------------------------------------------------------------
{
  assert (elt!=0); // can't divide by zero !

  matrix<Tf> mp = mat;
  mp /= elt;

  return mp;
}


//--------------------------------------------------------------------------------
template <class Tf> Tf operator | (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
//--------------------------------------------------------------------------------
{
  assert (mat1.nbRows==mat2.nbRows); // matrices must have the same size
  assert (mat1.nbRows);

  Tf sum=0;
  for (int i=1; i<=mat1.nbRows; i++)
    for (int j=1; j<=mat1.nbColumns; j++)
      sum += mat1(i,j) * mat2(i,j);

  return sum;
}


//----------------------------------------------------------------------------------
template <class Tf> matrix<Tf> operator ^ (const vector<Tf>& v, const vector<Tf>& w)
//----------------------------------------------------------------------------------
{
  assert (v.dim() && w.dim());

  matrix<Tf> mat(v.dim(),w.dim(),false); // allocates memory for columns (no initialization)
  for (int i=1; i<=v.dim(); i++)
    for (int j=1; j<=w.dim(); j++)
      mat (i,j) = v[i]*w[j];

  return mat;
}


//-------------------------------------------------------------------------
template <class T> matrix<T>& matrix<T>::operator += (const matrix<T>& mat)
//-------------------------------------------------------------------------
{
  assert(nbRows && nbColumns);
  assert ((nbRows==mat.nbRows) && (nbColumns==mat.nbColumns)); // matrices must have the same size

  // the following command allows the following operation : matrix += symatrix, so use it instead of mrows[i] += mat[i+1];
  for (int i=1; i<=nbRows; i++)
    for (int j=1; j<=nbColumns; j++)
      (*this)(i,j) += mat(i,j);

  return *this;
}


//-------------------------------------------------------------------------
template <class T> matrix<T>& matrix<T>::operator -= (const matrix<T>& mat)
//-------------------------------------------------------------------------
{
  assert(nbRows && nbColumns);
  assert ((nbRows==mat.nbRows) && (nbColumns==mat.nbColumns)); // matrices must have the same size

  // the following command allows the following operation : matrix += symatrix, so use it instead of mrows[i] += mat[i+1];
  for (int i=1; i<=nbRows; i++)
    for (int j=1; j<=nbColumns; j++)
      (*this)(i,j) -= mat(i,j);

  return *this;
}


//-------------------------------------------------------------------------
template <class T> matrix<T>& matrix<T>::operator *= (const matrix<T>& mat)
//-------------------------------------------------------------------------
{
  assert ((nbColumns==mat.nbRows)&&(nbColumns==mat.nbColumns));
  assert (nbColumns==(*this)[1].dim()); // this operator is not defined for symmatrix on the left side of the equation

  matrix<T> mt(nbRows,nbColumns); // allocates memory and set matrix components to zero

  for (int i=1; i<=nbRows; i++)
    for (int j=1; j<=mat.nbColumns; j++)
      for (int k=1; k<=mat.nbRows; k++)
	mt(i,j) += (*this)(i, k) * mat(k, j);
  // next lines are faster than *this = mt; because we use the temporary object (no copy needed)
  delete [] mrows;
  mrows = mt.mrows;
  mt.nbRows = mt.nbColumns = 0;

  return *this;
}


//-----------------------------------------------------------------
template <class T> matrix<T>& matrix<T>::operator *= (const T& elt)
//-----------------------------------------------------------------
{
  assert (nbRows && nbColumns);

  for (int i=1; i<=nbRows; i++)
      (*this)[i] *= elt;

  return *this;
}


//-------------------------------------------------------------------------
template <class T> matrix<T>& matrix<T>::operator /= (const matrix<T>& mat)
//-------------------------------------------------------------------------
{
  // a modifier et optimiser - cf operateur /
  assert (nbColumns==(*this)[1].dim()); // this operator is not defined for symmatrix on the left side of the equation

  return   *this *= gauss(mat);
}


//-----------------------------------------------------------------
template <class T> matrix<T>& matrix<T>::operator /= (const T& elt)
//-----------------------------------------------------------------
{
  assert (elt);

  for (int i=1; i<=nbRows; i++)
    (*this)[i] /= elt;

  return *this;
}


//-----------------------------------------------------------------------------------
template <class Tf> bool operator == (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
//-----------------------------------------------------------------------------------
{
  bool result=( (mat1.nbRows==mat2.nbRows) && (mat2.nbColumns==mat2.nbColumns) );

  for (int i=1; i<=mat1.nbRows && result; i++)
    for (int j=1; (j<=mat1.nbColumns) && (result); j++)
      if (abs (mat1(i,j)) < epsilon) result *= abs (mat2(i,j)) < epsilon;
      else result *= abs ((mat1(i,j) - mat2(i,j)) / mat1(i,j)) < epsilon;    // where epsilon is the computer approximation

  return result;
}


//-----------------------------------------------------------------------------------
template <class Tf> bool operator != (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
//-----------------------------------------------------------------------------------
{
  return !(mat1 == mat2);
}


//--------------------------------------------------------------------------
template <class Tf> ostream& operator << (ostream& s, const matrix<Tf>& mat)
//--------------------------------------------------------------------------
{
  for (int i=1; i<=mat.nbRows; i++)
      s << mat[i];
  s << endl;
  return s;
}


//--------------------------------------------------------------------------
template <class Tf> istream& operator >> (istream& s, const matrix<Tf>& mat)
//--------------------------------------------------------------------------
{
  for (int i=1; i<=mat.nbRows; i++)
      s >> mat[i];
  return s;
}


//----------------------------------------
template <class Tf> matrix<Tf> Id2 (int n)
//----------------------------------------
{
  assert (n > 0);
  matrix<Tf> I(n,n);

  for (int i=1; i<=n; i++)
    I(i,i) = 1;

  return I;
}


//------------------------------------------------
template <class Tf> Tf det (const matrix<Tf>& mat)
//------------------------------------------------
{
  assert (mat.nbRows == mat.nbColumns); // matrix must be squared
  Tf value = 0;
  if (mat.nbRows==1) return mat(1,1);
  matrix<Tf> m(mat.nbRows-1,mat.nbColumns-1);

  for (int i=1; i<=mat.nbRows; i++)
  {   for (int j=2; j<=mat.nbColumns; j++)
	  for (int k=1,l=1; k<=mat.nbRows; k++)
	      if (k!=i)
		 m(l++,j-1) = mat(k,j);
      if (mat(i,1))
      {
         if ( ((float)(i)/2) == int((float) (i)/2) )
	    value -= mat(i,1)*det(m);
	 else
	    value += mat(i,1)*det(m);
      }
  }
  return value;
}


//------------------------------------------------------
template <class Tf> matrix<Tf> t (const matrix<Tf>& mat)
//------------------------------------------------------
{
  assert (mat.nbRows && mat.nbColumns);
  matrix<Tf> mt(mat.nbColumns,mat.nbRows,false); // allocates memory (no initialization)
  for (int i=1; i<=mat.nbRows; i++)
      for (int j=1; j<=mat.nbColumns; j++)
	  mt(j,i) = mat(i,j);
  return mt;
}


//--------------------------------------------
template <class T> matrix<T> matrix<T>::inv ()
//--------------------------------------------
{
  assert (nbRows==nbColumns); // matrix must be squared
  assert (nbRows); // matrix must exists
  T dt = det(*this);
  matrix<T> mat(nbRows-1,nbColumns-1), newmat (nbRows,nbColumns);
  assert (dt); // matrix can't be inverted if the determinant = 0
  // computes the cofactors matrix and divides it by the determinant
  for (int i=1; i<=nbRows; i++)
      for (int j=1; j<=nbColumns; j++)
      {   for (int k=1,x=1; k<=nbRows; k++)
	      if (k!=i)
	      {	 for (int l=1,y=1; l<=nbColumns; l++)
		     if (l!=j)
			mat(x,y++) = (*this)(k,l);
		 x++;
	      }
	  if ( ((float)(i+j)/2) == int ((float)(i+j)/2) )
	     newmat(i,j) = det(mat)/dt;
	  else
	     newmat(i,j) = -det(mat)/dt;
      }
  // tranposes the cofactors matrix
  newmat = t(newmat);
  // divides the transposed cofactors matrix by the determinant
  //  newmat = newmat/dt;
  return newmat;
}


//---------------------------------------------------
template <class Tf> matrix<Tf> gauss (matrix<Tf> mat)
//---------------------------------------------------
{
  assert (mat.nbRows == mat.nbColumns); // matrix must be squared
  matrix<Tf> matinv =& Id2<Tf>(mat.nbRows); // initializes matinv as the identity matrix
  int posx, dim = mat.nbRows;
  Tf pivot, elt;

  for (int k=1; k<=dim; k++)
    { pivot = 0;
      // search the biggest element of the ith line and kth column
      for (int i=k; i<=dim; i++)
	if (abs(mat(i,k)) > abs(pivot))
	  { pivot = mat(i,k);
	    posx = i;
	  }

      assert (abs(pivot) > epsilon);

      // Rows permutation : row k <-> row posx
      if (posx != k)
      	{ mat.permute_rows(k,posx);
      	  matinv.permute_rows(k,posx);
      	}

      //  starts the resolution at step k
      vector<Tf> *mat_k, *matinv_k;
      mat_k = &mat[k];
      matinv_k = &matinv[k];
      Tf mat_kk = (*mat_k)[k];

      for (int i=1; i<k; i++)
      	{ vector<Tf> *mat_i, *matinv_i;
	  mat_i = &mat[i];
	  matinv_i = &matinv[i];
	  elt = (*mat_i)[k] / mat_kk;
	  for (int j=k+1; j<=dim; j++) (*mat_i)[j] -= (*mat_k)[j] * elt;
      	  for (int j=1; j<=dim; j++) (*matinv_i)[j] -= (*matinv_k)[j] * elt;
	}
      for (int i=k+1; i<=dim; i++)
	{ vector<Tf> *mat_i, *matinv_i;
	  mat_i = &mat[i];
	  matinv_i = &matinv[i];
	  elt = (*mat_i)[k] / mat_kk;
	  for (int j=k+1; j<=dim; j++) (*mat_i)[j] -= (*mat_k)[j] * elt;
	  for (int j=1;   j<=dim; j++) (*matinv_i)[j] -= (*matinv_k)[j] * elt;
	}
      for (int j=k+1; j<=dim; j++) (*mat_k)[j] /= mat_kk;
      for (int j=1; j<=dim; j++) (*matinv_k)[j] /= mat_kk;
    }

  return matinv;
}


//-----------------------------------------------------------------------
template <class Tf> bool inv (matrix<Tf>& mat, long double eps = epsilon)
//-----------------------------------------------------------------------
{
  assert (mat.nbRows == mat.nbColumns); // matrix must be squared
  matrix <Tf> matinv =& Id2<Tf>(mat.nbRows); // initializes matinv as the identity matrix
  matrix<Tf> mat_copy = mat;
  int posx, dim = mat.nbRows;
  Tf pivot, elt;

  for (int k=1; k<=dim; k++)
    { pivot = 0;
      // search the biggest element of the ith line and kth column
      for (int i=k; i<=dim; i++)
	if (abs(mat(i,k)) > abs(pivot))
	  { pivot = mat(i,k);
	    posx = i;
	  }

      if (abs(pivot) <= eps)
	{ mat = mat_copy;
	  return false;
	}

      // Rows permutation : row k <-> row posx
      if (posx != k)
      	{ mat.permute_rows(k,posx);
      	  matinv.permute_rows(k,posx);
      	}

      //  starts the resolution at step k
      vector<Tf> *mat_k, *matinv_k;
      mat_k = &mat[k];
      matinv_k = &matinv[k];
      Tf mat_kk = (*mat_k)[k];

      for (int i=1; i<k; i++)
      	{ vector<Tf> *mat_i, *matinv_i;
	  mat_i = &mat[i];
	  matinv_i = &matinv[i];
	  elt = (*mat_i)[k] / mat_kk;
	  for (int j=k+1; j<=dim; j++) (*mat_i)[j] -= (*mat_k)[j] * elt;
      	  for (int j=1; j<=dim; j++) (*matinv_i)[j] -= (*matinv_k)[j] * elt;
	}
      for (int i=k+1; i<=dim; i++)
	{ vector<Tf> *mat_i, *matinv_i;
	  mat_i = &mat[i];
	  matinv_i = &matinv[i];
	  elt = (*mat_i)[k] / mat_kk;
	  for (int j=k+1; j<=dim; j++) (*mat_i)[j] -= (*mat_k)[j] * elt;
	  for (int j=1;   j<=dim; j++) (*matinv_i)[j] -= (*matinv_k)[j] * elt;
	}
      for (int j=k+1; j<=dim; j++) (*mat_k)[j] /= mat_kk;
      for (int j=1; j<=dim; j++) (*matinv_k)[j] /= mat_kk;
    }

  mat =& matinv;

  //  if (eps != epsilon) assert (mat * mat_copy == Id2<Tf>(mat.Rows()));

  return true;
}

/* FAUX - A REVOIR
//---------------------------------------------------
template <class T> matrix<T> gauss_pt (matrix<T> mat)
//---------------------------------------------------
{
  assert (mat.nbRows == mat.nbColumns); // matrix must be squared
  matrix <T> matinv =& Id2<T>(mat.nbRows); // initializes matinv as the identity matrix
  int posx, posy, dim = mat.nbRows;
  T pivot, elt;

  for (int k=1; k<=dim; k++)
    { pivot = 0;
      // search the biggest element of the ith line and kth column
      for (int i=k; i<=dim; i++)
	for (int j=k; j<=dim; j++)
	  if (abs(mat(i,j)) > abs(pivot))
	  { pivot = mat(i,j);
	    posx = i;
	    posy = j;
	  }

      assert (abs(pivot)>epsilon);

      // Rows permutation : row k <-> row posx
      if (posx != k)
	{ mat.permute_rows(k,posx);
	  matinv.permute_rows(k,posx);
	}

      // Columns permutation : column k <-> column posy
      if (posy!=k)
	{ mat.permute_columns(k,posy);
	  matinv.permute_columns(k,posy);
	}

      //  starts the resolution at step k
      for (int i=1; i<k; i++)
	{ elt = mat(i,k) / mat(k,k);
	  for (int j=k+1; j<=dim; j++) mat(i,j) -= mat(k,j) * elt;
	  for (int j=1;   j<=dim; j++) matinv(i,j) -= matinv(k,j) * elt;
	}
      for (int i=k+1; i<=dim; i++)
	{  elt = mat(i,k) / mat(k,k);
	   for (int j=k+1; j<=dim; j++) mat(i,j) -= mat(k,j) * elt;
	   for (int j=1;   j<=dim; j++) matinv(i,j) -= matinv(k,j) * elt;
	}
      for (int j=k+1; j<=dim; j++) mat(k,j) /= mat(k,k);
      for (int j=1;   j<=dim; j++) matinv (k,j) /= mat(k,k);
    }

  return matinv;
}
*/


/*!
  \brief Computes the p-norm for matrix.

  Let \f$ A \f$ be a \f$ n \times m \f$ matrix. Then, its p-norm is given by: \n
  \f$ \left\| A \right\|_p = \displaystyle \left( \sum_{i=1}^{n} \sum_{j=1}^{m} \left| a_{ij} \right|^p \right)^{1/p} \f$.\n
  The special case p = 2 is the Frobenius norm also called the Hilbert-Schmidt norm.

  \param p value for p-norm
  \return \f$ \left\| A \right\|_p \f$

*/

//--------------------------------------------
template <class T> T matrix<T>::p_norm (int p)
//--------------------------------------------
{
  assert (p > 0);

  T sum = 0;
  for (int i=1; i<=nbRows; i++)
    for (int j=1; j<=nbColumns; j++)
      sum += pow (mrows[i-1][j], p);

  return pow (sum, (long double) 1/p);
}


/*!
  \brief Writes matrix to hard disk.

  \param path path and filename
*/

//--------------------------------------------------------
template <class T> void matrix<T>::save (const char* path)
//--------------------------------------------------------
{
  ofstream file(path, ios::out);
  assert (!file.fail());
  file << *this;
  file.close();
}


/*!
  \brief Writes matrix to hard disk.

  \param filename path and filename
*/

//--------------------------------------------------------------
template <class T> void matrix<T>::save (const string& filename)
//--------------------------------------------------------------
{
  const char *path = filename.c_str();
  save (path);
}


/*!
  \brief Reads matrix from hard disk.

  \param path path and filename
*/

//--------------------------------------------------------
template <class T> void matrix<T>::read (const char* path)
//--------------------------------------------------------
{
  ifstream file(path, ios::in);
  assert (!file.fail());
  file >> *this;
  file.close();
}


/*!
  \brief Reads matrix from hard disk.

  \param filename path and filename
*/

//--------------------------------------------------------------
template <class T> void matrix<T>::read (const string& filename)
//--------------------------------------------------------------
{
  const char *path = filename.c_str();
  read (path);
}


//-------------------------------------------------------
template <class T> matrix<T>& matrix<T>::approximation ()
//-------------------------------------------------------
{
  assert (nbRows && nbColumns);

  T elt;
  T norm = sup (p_norm(2), (T) 1);

  for (int i=1; i<=nbRows; i++)
    for (int j=1; j<=nbColumns; j++)
      if (abs (mrows[i-1][j]) < epsilon * norm)
	mrows[i-1][j] = 0;

  return *this;
}


//----------------------------------------------
template <class Tf> matrix<Tf> kronecker (int n)
//----------------------------------------------
{
  assert (n > 0);
  return Id2<Tf>(n);
}


//------------------------------------------------------------
template <class T> void matrix<T>::permute_rows (int i, int j)
//------------------------------------------------------------
{  
  assert (nbColumns==(*this)[1].dim()); // does not allow permutation for symmatrix or sparse matrix
  assert (i>0 && i<=nbRows);
  assert (j>0 && j<=nbRows);

  if (i!=j) permute (mrows[i-1],mrows[j-1]);
}


//---------------------------------------------------------------
template <class T> void matrix<T>::permute_columns (int i, int j)
//---------------------------------------------------------------
{  
  assert (nbColumns==(*this)[1].dim()); // does not allow permutation for symmatrix or sparse matrix
  assert (i>0 && i<=nbColumns);
  assert (j>0 && j<=nbColumns);
  T elt;

  if (i!=j)
    for (int k=1; k<=nbRows; k++)
      { elt=(*this)(k,i);
        (*this)(k,i)=(*this)(k,j);
	(*this)(k,j)=elt;
      }
}


//--------------------------------------------------------
template <class Tf> matrix<Tf> abs (const matrix<Tf>& mat)
//--------------------------------------------------------
{
  assert (mat.nbColumns);
  matrix<Tf> mtx;
  mtx.create(mat.nbRows,mat.nbColumns);

  for (int i=1; i<=mat.nbRows; i++)
    mtx[i] =& abs(mat[i]);

  return mtx;
}


//------------------------------------------------
template <class Tf> Tf max (const matrix<Tf>& mat)
//------------------------------------------------
{
  assert (mat.nbColumns);
  Tf val = mat(1,1);

  for (int i=1; i<=mat.nbRows; i++)
    for (int j=1; j<=mat.nbColumns; j++)
      if (val<mat(i,j)) val = mat(i,j);

  return val;
}


//-----------------------------------------------
template <class Tf> Tf tr (const matrix<Tf>& mat)
//-----------------------------------------------
{ 
  assert (mat.nbRows);
  assert (mat.nbRows==mat.nbColumns); // matrix must be squared

  Tf val = 0;

  for (int i=1; i<=mat.nbRows; i++)
    val += mat(i,i);

  return val;
}


//-------------------------------------------------------------
template <class Tf> vector<Tf> diagonal (const matrix<Tf>& mat)
//-------------------------------------------------------------
{
  assert (mat.nbRows);
  assert (mat.Rows()==mat.Columns());

  vector<Tf> v(mat.nbRows,false);

  for (int i=1; i<=mat.nbRows; i++)
    v[i] = mat(i,i);

  return v;
}


//------------------------------------------------------------------------------------------
template <class Tf> matrix<Tf> submatrix (const matrix<Tf>& mat, int m, int n, int p, int q)
//------------------------------------------------------------------------------------------
{
  assert (m>0 && m<=n && n<=mat.nbRows);
  assert (p>0 && p<=q && q<=mat.nbColumns);

  matrix<Tf> sub_mat;
  sub_mat.create (n-m+1,q-p+1);

  for (int i=m; i<=n; i++)
    sub_mat[i-m+1] =& subvector(mat[i],p,q);

  return sub_mat;
}


//----------------------------------------------------------------------------------------
template <class Tf> matrix<Tf> merge_rows (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
//----------------------------------------------------------------------------------------
{
  assert (mat1.nbColumns==mat1[1].dim()); // excludes symmatrix
  assert (mat1.nbColumns == mat2.nbColumns);

  matrix<Tf> mat;
  mat.create (mat1.nbRows+mat2.nbRows,mat1.nbColumns);

  for (int i=1; i<=mat1.nbRows; i++)
    mat[i] = mat1[i];
  for (int i=1; i<=mat2.nbRows; i++)
    mat[mat1.nbRows+i] = mat2[i];

  return mat;
}


//-------------------------------------------------------------------------------------------
template <class Tf> matrix<Tf> merge_columns (const matrix<Tf>& mat1, const matrix<Tf>& mat2)
//-------------------------------------------------------------------------------------------
{
  assert (mat1.nbColumns==mat1[1].dim()); // excludes symmatrix
  assert (mat1.nbRows == mat2.nbRows);

  matrix<Tf> mat;
  mat.create (mat1.nbRows,mat1.nbColumns+mat2.nbColumns);

  for (int i=1; i<=mat.nbRows; i++)
    mat[i] =& merge(mat1[i],mat2[i]);

  return mat;
}


}


#endif
