/***************************************************************
 *                    simula.plus@cemes.fr                     *
 *                   GNU/linux version 3.4.0                   *
 *            software under General Public License            *
 ***************************************************************
 * copyright © 2004,2005,2006,2007,2008,2009,2012 COLLARD Christophe
 * copyright © 2004,2005,2006,2007,2008,2009,2012 Centre National de la Recherche Scientifique
 * copyright © 2004,2005,2006,2007,2008,2009 Arts et Métiers ParisTech
 * copyright © 2004,2005,2006,2007 Université de Valenciennes et du Hainaut-Cambrésis
 * copyright © 2004,2005,2006,2007,2008,2009 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
 * copyright © 2004,2005,2006,2007 Laboratoire de Mathématiques et ses Applications de Valenciennes (LAMAV)
 * copyright © 2012 Centre d'Elaboration de Matériaux et d'Etudes Structurales (CEMES - CNRS)
 ***************************************************************/

/*
    maths-test belongs to Mathematical Object Libraries (MOL++)
    MOL++ is part of Simula+

    Simula+ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Simula+ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Simula+; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef __cplusplus
#error Must use C++ for maths
#endif

#ifndef _maths_test_hpp
#define _maths_test_hpp


#ifndef _iostream
#include <iostream>
#endif

#ifndef _colors_hpp
#include "colors.hpp"
#endif

#ifndef _affiche_hpp
#include "tests/affiche.hpp"
#endif

#ifndef _parameters_h
#include "parameters.h"
#endif

#ifndef _maths_hpp
#include "MOL++/maths.hpp"
#endif

using namespace mol;


//==========================
bool test_maths (int detail)
//==========================
{
  bool result = true;

  long double x= 5.2375, y = -5.2375;

  if (detail) affiche ("abs", abs(y)==x);
  else result *= (abs(y)==x);

  if (detail) affiche ("power", abs(power(x,2)-x*x)<epsilon);
  else result *= (abs(power(x,2)-x*x)<epsilon);

  if (detail) affiche ("power", abs(power(x,2)-x*x)<epsilon);
  else result *= (abs(power(x,2)-x*x)<epsilon);

  if (detail) affiche ("sgn", sgn(x)==1 && sgn(-x)==-1);
  else result *= (sgn(x)==1 && sgn(-x)==-1);

  if (detail) affiche ("sup", sup(x,y) == x && sup(abs(x),abs(y))==x);
  else result *= (sup(x,y) == x && sup(abs(x),abs(y))==x);

  if (detail) affiche ("inf", inf(x,y) == y && inf(abs(x),abs(y))==x);
  else result *= (inf(x,y) == y && inf(abs(x),abs(y))==x);

  if (detail) affiche ("pi", abs(pi()-3.14159265358979323846264338327950288419716939937510) < epsilon);
  else result *= (abs(pi()-3.14159265358979323846264338327950288419716939937510)<epsilon);

  long double reel = 9*8*7*6*5*4*3*2;
  if (detail) affiche ("factorial", factorial(9)==reel);
  else result *= (factorial(9)==reel);

  if (detail) affiche ("inverse factorial", inv_factorial(reel)==9 && inv_factorial(x)==-1);
  else result *= (inv_factorial(reel)==9 && inv_factorial(x)==-1);

  long double a, b, c, d, e;
  a = 0; b = 0.5*epsilon;
  c = 1.E15;
  d = c * (1+0.5*epsilon);
  e = c * (1+epsilon);
  if (detail) affiche ("relative error", relative_error(a,b) && relative_error(c,d) && !relative_error(c,e));
  else result *= (relative_error(a,b) && relative_error(c,d) && !relative_error(c,e));

  cout << endl;

  cout << "============================================================== \n";
  if (result) cout << green << "                      maths test passed" << reset;
  else cout << red << "                      maths test failed" << reset;
  cout << "============================================================== \n";

  return result;
}



#endif
